/*
** ============================================================================
**
** FILE
**  $Id: pic18_timers.c,v 1.0 2007/06/20 krk Exp apache $
**
** DESCRIPTION
**  Contains the timer specific functions
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#include "pic18_timers.h"

/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/
volatile uint8 SystemTimerWriting;
volatile uint16 SystemTimer;
/*------------------------------------------------------------------------*/
/*						LOCAL function prototypes						  */
/*------------------------------------------------------------------------*/

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void StartTmr0(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable)
  +
  + DESCRIPTION:    start TMR0 in the specified mode
  +
  +	INPUT:			operation_mode: TMR0_8BIT / TMR0_16BIT
  +					prescaler: 		TMR0_256 ... TMR0_1
  +					period:			the duration of the timing	
  +					it_enable:		TRUE - it enables the IT; FALSE - it disables 
  +									the IT (status should be polled)	
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#pragma interrupt_level 1
void StartTmr0(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable)
{
	//setup timer
	T0CON = TMR0_ENABLE | operation_mode | prescaler;

	//set time period
	if( operation_mode == TMR0_8BIT )
	{
		TMR0L = 255 - (uint8)(period & 0x00FF);
	}
	else
	{
		period = 65535 - period;
		TMR0H = (uint8)(period >> 8);
		TMR0L = (uint8)(period & 0x00FF);
	}

	//enable IT if needed
	if( it_enable == TRUE )
	{
		EnableTmr0It();
	}
	else
	{
		DisableTmr0It();
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint16 GetTmr0(uint8 operation_mode)
  +
  + DESCRIPTION:    gives back the actual value of TMR0
  +
  +	INPUT:			operation_mode: TMR0_8BIT / TMR0_16BIT
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint16 GetTmr0(uint8 operation_mode)
{
	uint16 Temp16;

	if( operation_mode == TMR0_8BIT )
	{
		return ((uint16)TMR0L);
	}
	else
	{
		Temp16 = TMR0L;
		Temp16 += (uint16)(TMR0H << 8);
		return Temp16;
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 Tmr0Expired(void)
  +
  + DESCRIPTION:    returns with TRUE if TMR0 already expired, 
  +					otherwise returns with FALSE
  +
  +	INPUT:			None
  +
  + RETURN:         TRUE / FALSE
  +
  + NOTES:          it doesn't clear the IT status flag!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 Tmr0Expired(void)
{
	if( TMR0IF == 0)
		return FALSE;
	else
		return TRUE;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void StartTmr1(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable)
  +
  + DESCRIPTION:    start TMR1 in the specified mode
  +
  +	INPUT:			operation_mode: TMR1_INTERNAL_ENABLE / TMR1_OSC_ENABLE
  +					prescaler: 		TMR1_8 ... TMR1_1
  +					period:			the duration of the timing	
  +					it_enable:		TRUE - it enables the IT; FALSE - it disables 
  +									the IT (status should be polled)	
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void StartTmr1(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable)
{
	//setup timer
	T1CON = operation_mode | prescaler;
	
	//set period
	period = 65535 - period;
	TMR1H = (uint8)(period >> 8);
	TMR1L = (uint8)(period & 0x00FF);

	//enable IT if needed
	if( it_enable == TRUE )
	{
		EnableTmr1It();
	}
	else
	{
		DisableTmr1It();
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint16 GetTmr1(void)
  +
  + DESCRIPTION:    gives back the actual value of TMR1
  +
  +	INPUT:			
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint16 GetTmr1(void)
{
	uint16 Temp16;

	Temp16 = TMR1L;
	Temp16 += (uint16)(TMR1H << 8);
	return Temp16;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 Tmr1Expired(void)
  +
  + DESCRIPTION:    returns with TRUE if TMR1 already expired, 
  +					otherwise returns with FALSE
  +
  +	INPUT:			None
  +
  + RETURN:         TRUE / FALSE
  +
  + NOTES:          it doesn't clear the IT status flag!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 Tmr1Expired(void)
{
	if( TMR1IF == 0)
		return FALSE;
	else
		return TRUE;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void StartTmr2(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable)
  +
  + DESCRIPTION:    start TMR2 in the specified mode
  +
  +	INPUT:			prescaler: 		TMR2_16 ... TMR2_1
  +					period:			the duration of the timing	
  +					it_enable:		TRUE - it enables the IT; FALSE - it disables 
  +									the IT (status should be polled)	
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void StartTmr2(uint8 prescaler, uint8 period, uint8 it_enable)
{
	//setuo timer
	T2CON = TMR2_ENABLE | prescaler;

	//set period
	TMR2 = 255 - period;

	//enable IT if needed
	if( it_enable == TRUE )
	{
		EnableTmr2It();
	}
	else
	{
		DisableTmr2It();
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 GetTmr2(void)
  +
  + DESCRIPTION:    gives back the actual value of TMR2
  +
  +	INPUT:		
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 GetTmr2(void)
{
	return TMR2;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 Tmr2Expired(void)
  +
  + DESCRIPTION:    returns with TRUE if TMR2 already expired, 
  +					otherwise returns with FALSE
  +
  +	INPUT:			None
  +
  + RETURN:         TRUE / FALSE
  +
  + NOTES:          it doesn't clear the IT status flag!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 Tmr2Expired(void)
{
	if( TMR2IF == 0)
		return FALSE;
	else
		return TRUE;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void StartTmr3(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable)
  +
  + DESCRIPTION:    start TMR3 in the specified mode
  +
  +	INPUT:			operation_mode: TMR3_INTERNAL_ENABLE / TMR3_OSC_ENABLE
  +					prescaler: 		TMR3_8 ... TMR3_1
  +					period:			the duration of the timing	
  +					it_enable:		TRUE - it enables the IT; FALSE - it disables 
  +									the IT (status should be polled)	
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void StartTmr3(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable)
{
	//setup timer
	T3CON = operation_mode | prescaler;

	//set period
	period = 65535 - period;
	TMR3H = (uint8)(period >> 8);
	TMR3L = (uint8)(period & 0x00FF);

	//enable IT if needed
	if( it_enable == TRUE )
	{
		EnableTmr3It();
	}
	else
	{
		DisableTmr3It();
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint16 GetTmr3(void)
  +
  + DESCRIPTION:    gives back the actual value of TMR3
  +
  +	INPUT:			
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint16 GetTmr3(void)
{
	uint16 Temp16;

	Temp16 = TMR3L;
	Temp16 += (uint16)(TMR3H << 8);
	return Temp16;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 Tmr3Expired(void)
  +
  + DESCRIPTION:    returns with TRUE if TMR3 already expired, 
  +					otherwise returns with FALSE
  +
  +	INPUT:			None
  +
  + RETURN:         TRUE / FALSE
  +
  + NOTES:          it doesn't clear the IT status flag!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 Tmr3Expired(void)
{
	if( TMR3IF == 0)
		return FALSE;
	else
		return TRUE;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void StartSystemTimer(void)
  +
  + DESCRIPTION:    start the system timer
  +
  + RETURN:         
  +
  + NOTES:          - system timer uses TMR1 with 32.768kHz crystal, with 1:1
  +					  prescaler.
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void StartSystemTimer(void)
{
	StartTmr1(TMR1_OSC_ENABLE, TMR1_1, 65535, TRUE);
	SystemTimer = 0;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void IncrementSystemTimer(void)
  +
  + DESCRIPTION:    increment the system timer
  +
  + RETURN:         
  +
  + NOTES:          - system timer uses TMR1 with 32.768kHz crystal, with 1:1
  +					  prescaler.
  +					- this function is called from the TMR1 interrupt routine
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void IncrementSystemTimer(void)
{
	SystemTimer++;
	SystemTimerWriting++;	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint32 GetSystemTimer(void)
  +
  + DESCRIPTION:    it gives back the actual value of the system timer
  +
  + RETURN:         value of the system timer (32 bit unsigned int)
  +
  + NOTES:          - lower 16 bits of the System Timer is the actual value of TMR1
  +					- upper 16 bits is a counter which is incremented at TMR1 overrun
  +					- resolution of the System Timer is ~30.51us
  +					- System timer overrun after ~36.4hours
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint32 GetSystemTimer(void)
{
	uint16 Temp16,tmr_value;
	uint8  OldSystemTimerWriting;
	uint32 Temp32;
	do
	{
		OldSystemTimerWriting = SystemTimerWriting;
		tmr_value = GetTmr1();
		Temp16 = SystemTimer;
	}while(OldSystemTimerWriting != SystemTimerWriting);
	//if there was no incrementing during reading then return, otherwise get new value
	Temp32 = (uint32)Temp16;
	Temp32 <<= 16;
	Temp32 += (uint32)tmr_value;
	return Temp32;
}
